import paho.mqtt.client as paho
import time

SUBSCRIBE = 'PlayRobotAIoT'
BROKER = 'broker.hivemq.com'
CLIENT = 'default'
client = ''
ACTION = ''

def on_message(client, userdata, message):
    time.sleep(1)
    mqttmsg = str(message.payload.decode("utf-8"))
    print("received message =",mqttmsg)
    ACTION(mqttmsg)
    
def send(publish,text):
    client.publish(publish,text)
    
def setProperty(name,action,subscribe = 'PlayRobotAIoT',broker = 'broker.hivemq.com'):
    global SUBSCRIBE,BROKER,CLIENT,ACTION
    SUBSCRIBE = subscribe
    BROKER = broker
    CLIENT = name
    ACTION = action
    
def connect():
    global client,CLIENT,ACTION
    if CLIENT == 'default' or ACTION == '':
        raise BaseException('please using MQTT.setProperty to assign ACTION and CLIENT')
    client = paho.Client(CLIENT)
    ###########MQTT部分###################
    #MQTT client name，注意同名稱會搶線問題
    client.on_message=on_message
    print("connecting to broker ",BROKER)
    client.connect(BROKER)#connect
    client.loop_start() #start loop to process received messages
    
    print("subscribing ")
    
    #subscribe的頻道名稱
    client.subscribe(SUBSCRIBE)
    



from playrobot import Servo
from keras.models import Sequential
from keras.layers import Dense,Dropout,Flatten,Conv2D,MaxPooling2D

import cv2
import serial
import time

class PiBB:
    def __init__(self):
        self.Speed = 50
        self.visionThread = 100
        self.model = ''
        self.detectPoint =  [(160,40),(200,40),(240,40),(280,40),(319,40)]
        self.colorThread = 150
        
    def dnnInit(self,path):
        model = Sequential()
        model.add(Conv2D(filters=16,kernel_size=(3,3),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(Conv2D(filters=48,kernel_size=(1,1),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(MaxPooling2D(pool_size=(2,2)))
        model.add(Conv2D(filters=24,kernel_size=(3,3),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(Conv2D(filters=24,kernel_size=(3,3),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(MaxPooling2D(pool_size=(2,2)))
        model.add(Dropout(0.25))
        model.add(Flatten())
        model.add(Dense(64, activation='relu'))
        model.add(Dense(64, activation='relu'))
        model.add(Dense(32, activation='relu'))
        model.add(Dense(32, activation='relu'))
        model.add(Dense(16, activation='relu'))
        model.add(Dense(16, activation='relu'))
        model.add(Dropout(0.25))
        model.add(Dense(2, activation='softmax'))
        model.load_weights(path)

        gray = cv2.imread('bird_binary.jpg',cv2.IMREAD_GRAYSCALE)
        test = cv2.resize(gray,(80,80),interpolation=cv2.INTER_CUBIC).reshape(1, 80, 80, 1).astype('float32')
        test_norm = test/255
        model.predict_classes(test_norm)
        
        self.model = model
        
    def rcCtrl(self,act):
        turn = {1:[self.speed,self.speed],
               2:[self.speed,self.speed],
               3:[self.speed,self.speed],
               4:[self.speed,self.speed]}
        Servo.setServoPulse(0,1500+turn[act][0])
        Servo.setServoPulse(1,1500+turn[act][1])

    def roadTrace(self,img,decide = 0):
        gray = cv2.threshold(cv2.cvtColor(img, cv2.COLOR_BGR2GRAY),self.visionThread,255,cv2.THRESH_BINARY)[1]
        lineState = ''
        for y,x in self.detectPoint:
            lineState += '0' if gray[x][y] == 255 else '1'
            
        for i,j in enumerate(self.detectPoint):
            cv2.putText(img,str(i),j,cv2.FONT_HERSHEY_COMPLEX,1,(0,150,255),1,cv2.LINE_AA)
        
        if lineState == '00000':decide = -1
        elif lineState[:4] == '1111':decide = 2
        elif lineState[1] == '1':decide = 1
        elif lineState[3] == '1':decide = -1
        cv2.putText(img,'right' if decide < 0 else 'left' if decide > 0 else 'front',
                    (50,50),cv2.FONT_HERSHEY_COMPLEX,1,(0,150,255),1,cv2.LINE_AA)
        tempColor = self.colorDetect(img)
        if tempColor == 'RED':
            colorToSound = {'RED':'紅色','BLUE':'藍色','GREEN':'綠色'}
            print('偵測到' + colorToSound[tempColor])
            #TTS.wordToSound('偵測到' + colorToSound[COLOR_DEC])
    
        return img,decide

    def autoDrive(self,img):
        decide = 0
        gray = cv2.threshold(cv2.cvtColor(img, cv2.COLOR_BGR2GRAY),self.visionThread,255,cv2.THRESH_BINARY)[1][85:235,80:230]
        cv2.rectangle(img, (85,80), (235, 230), (0, 255, 0), 2)
        showText = ''
    
        test = cv2.resize(gray,(80,80),interpolation=cv2.INTER_CUBIC).reshape(1, 80, 80, 1).astype('float32')
        test_norm = test/255
        self.model.predict_classes(test_norm) == 0
            
        cv2.putText(img,showText,(200,50),
            cv2.FONT_HERSHEY_COMPLEX,1,(0,150,255),1,cv2.LINE_AA)
        
        img,decide = self.roadTrace(img)
        if decide == -1:self.rcCtrl(2)
        elif decide == 1:self.rcCtrl(3)
        elif decide == 2:self.rcCtrl(5)
        else:self.rcCtrl(1)
        return img

    def colorDetect(self,img):
        COLOR =  self.colorThread
        height, width = img.shape[:2]
        height, width = height//2, width//2
        B, G, R = cv2.split(img)
        B, G, R = B[height][width], G[height][width], R[height][width]
        if G > COLOR and B < COLOR and R < COLOR:
            return 'GREEN'
        elif B > COLOR and G < COLOR and R < COLOR:
            return 'BLUE'
        elif R > COLOR and G < COLOR and B < COLOR:
            return 'RED'
        else:
            return 'WHITE'
        
    def run(self):
        self.rcCtrl(1)
        cap = cv2.VideoCapture(0)
        # 設定影像尺寸
        cap.set(3,320)
        cap.set(4,240)
        while True:
            ret, frame = cap.read()
            if ret:
                height, width = frame.shape[:2]
                rgbImage = self.autoDrive(frame)
                cv2.imshow('cam',rgbImage)
                if cv2.waitKey(1) & 0xFF == ord('q'):
                    cv2.destroyAllWindows()
                    cap.release()
                    self.rcCtrl(8,0)
                    break

class AIoTCar:
    def __init__(self):
        self.Speed = 50
        self.visionThread = 100
        self.model = ''
        self.detectPoint =  [(160,40),(200,40),(240,40),(280,40),(319,40)]
        self.colorThread = 150
        self.ser = serial.Serial('/dev/ttyACM0',115200,timeout=1)
        
    def dnnInit(self,path):
        model = Sequential()
        model.add(Conv2D(filters=16,kernel_size=(3,3),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(Conv2D(filters=48,kernel_size=(1,1),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(MaxPooling2D(pool_size=(2,2)))
        model.add(Conv2D(filters=24,kernel_size=(3,3),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(Conv2D(filters=24,kernel_size=(3,3),padding='same',input_shape=(80,80,1),activation='relu'))
        model.add(MaxPooling2D(pool_size=(2,2)))
        model.add(Dropout(0.25))
        model.add(Flatten())
        model.add(Dense(64, activation='relu'))
        model.add(Dense(64, activation='relu'))
        model.add(Dense(32, activation='relu'))
        model.add(Dense(32, activation='relu'))
        model.add(Dense(16, activation='relu'))
        model.add(Dense(16, activation='relu'))
        model.add(Dropout(0.25))
        model.add(Dense(2, activation='softmax'))
        model.load_weights(path)

        gray = cv2.imread('bird_binary.jpg',cv2.IMREAD_GRAYSCALE)
        test = cv2.resize(gray,(80,80),interpolation=cv2.INTER_CUBIC).reshape(1, 80, 80, 1).astype('float32')
        test_norm = test/255
        model.predict_classes(test_norm)
        
        self.model = model
        
    def rcCtrl(self,act):
        self.ser.flush()
        nowCmd = str(act) + '_' + str(self.speed) + '\n'
        self.ser.write(nowCmd.encode())
        
    def roadTrace(self,img,decide = 0):
        global EAT_TIME
        detectPoint = [(160,40),(200,40),(240,40),(280,40),(319,40)]
        gray = cv2.threshold(cv2.cvtColor(img, cv2.COLOR_BGR2GRAY),self.visionThread,255,cv2.THRESH_BINARY)[1]
        lineState = ''
        for y,x in detectPoint:
            lineState += '0' if gray[x][y] == 255 else '1'
            
        for i,j in enumerate(detectPoint):
            cv2.putText(img,str(i),j,cv2.FONT_HERSHEY_COMPLEX,1,(0,150,255),1,cv2.LINE_AA)
        
        if lineState == '00000':decide = -1
        elif lineState[:4] == '1111':decide = 2
        elif lineState[1] == '1':decide = 1
        elif lineState[3] == '1':decide = -1
        cv2.putText(img,'right' if decide < 0 else 'left' if decide > 0 else 'front',
                    (50,50),cv2.FONT_HERSHEY_COMPLEX,1,(0,150,255),1,cv2.LINE_AA)
        tempColor = self.colorDetect(img)
        if tempColor == 'RED':
            colorToSound = {'RED':'紅色','BLUE':'藍色','GREEN':'綠色'}
            print('偵測到' + colorToSound[tempColor])
            #TTS.wordToSound('偵測到' + colorToSound[COLOR_DEC])
        return img,decide
    
    def autoDrive(self,img):
        global EAT_TIME,STATE,image
        decide = 0
        gray = cv2.threshold(cv2.cvtColor(img, cv2.COLOR_BGR2GRAY),self.visionThread,255,cv2.THRESH_BINARY)[1][85:235,80:230]
        cv2.rectangle(img, (85,80), (235, 230), (0, 255, 0), 2)
        showText = ''
        if time.time() - EAT_TIME > 30:
            test = cv2.resize(gray,(80,80),interpolation=cv2.INTER_CUBIC).reshape(1, 80, 80, 1).astype('float32')
            test_norm = test/255
            if self.model.predict_classes(test_norm) == 0:
                EAT_TIME = time.time()
                showText = 'food'
                STATE = 'food'
            else:
                showText = 'no food'
        else:
            showText = 'full'
            
        if time.time() - EAT_TIME > 60:
            STATE = 'nofood'
            showText = 'hungary'
            
        cv2.putText(img,showText,(200,50),
            cv2.FONT_HERSHEY_COMPLEX,1,(0,150,255),1,cv2.LINE_AA)
        
        img,decide = self.roadTrace(img)
        if STATE != 'nofood':
            if decide == -1:self.rcCtrl(2,self.speed)
            elif decide == 1:self.rcCtrl(3,self.speed)
            elif decide == 2:
                self.rcCtrl(5,self.speed)
            else:self.rcCtrl(1,self.speed)
        else:
            self.rcCtrl(8,self.speed)
        return img
    
    def colorDetect(self,img):
        COLOR = self.colorThread 
        height, width = img.shape[:2]
        height, width = height//2, width//2
        B, G, R = cv2.split(img)
        B, G, R = B[height][width], G[height][width], R[height][width]
        if G > COLOR and B < COLOR and R < COLOR:
            return 'GREEN'
        elif B > COLOR and G < COLOR and R < COLOR:
            return 'BLUE'
        elif R > COLOR and G < COLOR and B < COLOR:
            return 'RED'
        else:
            return 'WHITE'
        
    def run(self):
        global STATE
        self.rcCtrl(1,self.speed)
        cap = cv2.VideoCapture(0)
        # 設定影像尺寸
        cap.set(3,320)
        cap.set(4,240)
        while True:
            ret, frame = cap.read()
            if ret:
                height, width = frame.shape[:2]
                rgbImage = self.autoDrive(frame)
                cv2.imshow('cam',rgbImage)
                if cv2.waitKey(1) & 0xFF == ord('q'):
                    cv2.destroyAllWindows()
                    cap.release()
                    self.rcCtrl(8,0)
                    break


from gtts import gTTS
from gtts_token.gtts_token import Token
from pygame import mixer
import calendar
import time
import math
import re
import requests

LANGUAGE = 'zh-tw'

#語音輸出
def _patch_faulty_function(self):
    if self.token_key is not None:
        return self.token_key

    timestamp = calendar.timegm(time.gmtime())
    hours = int(math.floor(timestamp / 3600))

    results = requests.get("https://translate.google.com/")
    tkk_expr = re.search("(tkk:*?'\d{2,}.\d{3,}')", results.text).group(1)
    tkk = re.search("(\d{5,}.\d{6,})", tkk_expr).group(1)
    
    a , b = tkk.split('.')

    result = str(hours) + "." + str(int(a) + int(b))
    self.token_key = result
    return result

Token._get_token_key = _patch_faulty_function

def wordToSound(text):
    global LANGUAGE
    file_name ='test2.mp3'
    tts = gTTS(text, lang = LANGUAGE)
    tts.save(file_name)
    mixer.init()
    mixer.music.load(file_name)
    mixer.music.play()
    while mixer.music.get_busy() == True:
        continue
    mixer.music.stop()
    mixer.quit()



#!/usr/bin/python

from .Raspi_PWM_Servo_Driver import PWM
import time

class Raspi_StepperMotor:
	MICROSTEPS = 8
	MICROSTEP_CURVE = [0, 50, 98, 142, 180, 212, 236, 250, 255]

	#MICROSTEPS = 16
	# a sinusoidal curve NOT LINEAR!
	#MICROSTEP_CURVE = [0, 25, 50, 74, 98, 120, 141, 162, 180, 197, 212, 225, 236, 244, 250, 253, 255]
	
	def __init__(self, controller, num, steps=200):
		self.MC = controller
		self.revsteps = steps
		self.motornum = num
		self.sec_per_step = 0.1
		self.steppingcounter = 0
		self.currentstep = 0

		num -= 1

		if (num == 0):
			self.PWMA = 8
			self.AIN2 = 9
			self.AIN1 = 10
			self.PWMB = 13
			self.BIN2 = 12
			self.BIN1 = 11
		elif (num == 1):
			self.PWMA = 2
			self.AIN2 = 3
			self.AIN1 = 4
			self.PWMB = 7
			self.BIN2 = 6
			self.BIN1 = 5
		else:
			raise NameError('MotorHAT Stepper must be between 1 and 2 inclusive')

	def setSpeed(self, rpm):
		self.sec_per_step = 60.0 / (self.revsteps * rpm)
		self.steppingcounter = 0

	def oneStep(self, dir, style):
		pwm_a = pwm_b = 255

		# first determine what sort of stepping procedure we're up to
		if (style == Raspi_MotorHAT.SINGLE):
    			if ((self.currentstep/(self.MICROSTEPS/2)) % 2): 
				# we're at an odd step, weird
				if (dir == Raspi_MotorHAT.FORWARD):
					self.currentstep += self.MICROSTEPS/2
				else:
					self.currentstep -= self.MICROSTEPS/2
			else:
				# go to next even step
				if (dir == Raspi_MotorHAT.FORWARD):
					self.currentstep += self.MICROSTEPS
				else:
					self.currentstep -= self.MICROSTEPS
		if (style == Raspi_MotorHAT.DOUBLE):
			if not (self.currentstep/(self.MICROSTEPS/2) % 2):
				# we're at an even step, weird
				if (dir == Raspi_MotorHAT.FORWARD):
					self.currentstep += self.MICROSTEPS/2
				else:
					self.currentstep -= self.MICROSTEPS/2
			else:
				# go to next odd step
				if (dir == Raspi_MotorHAT.FORWARD):
					self.currentstep += self.MICROSTEPS
				else:
					self.currentstep -= self.MICROSTEPS
		if (style == Raspi_MotorHAT.INTERLEAVE):
			if (dir == Raspi_MotorHAT.FORWARD):
				self.currentstep += self.MICROSTEPS/2
			else:
				self.currentstep -= self.MICROSTEPS/2

		if (style == Raspi_MotorHAT.MICROSTEP):
			if (dir == Raspi_MotorHAT.FORWARD):
				self.currentstep += 1
			else:
				self.currentstep -= 1

                	# go to next 'step' and wrap around
                	self.currentstep += self.MICROSTEPS * 4
                	self.currentstep %= self.MICROSTEPS * 4

			pwm_a = pwm_b = 0
			if (self.currentstep >= 0) and (self.currentstep < self.MICROSTEPS):
				pwm_a = self.MICROSTEP_CURVE[self.MICROSTEPS - self.currentstep]
				pwm_b = self.MICROSTEP_CURVE[self.currentstep]
			elif (self.currentstep >= self.MICROSTEPS) and (self.currentstep < self.MICROSTEPS*2):
				pwm_a = self.MICROSTEP_CURVE[self.currentstep - self.MICROSTEPS]
				pwm_b = self.MICROSTEP_CURVE[self.MICROSTEPS*2 - self.currentstep]
			elif (self.currentstep >= self.MICROSTEPS*2) and (self.currentstep < self.MICROSTEPS*3):
				pwm_a = self.MICROSTEP_CURVE[self.MICROSTEPS*3 - self.currentstep]
				pwm_b = self.MICROSTEP_CURVE[self.currentstep - self.MICROSTEPS*2]
			elif (self.currentstep >= self.MICROSTEPS*3) and (self.currentstep < self.MICROSTEPS*4):
                                pwm_a = self.MICROSTEP_CURVE[self.currentstep - self.MICROSTEPS*3]
                                pwm_b = self.MICROSTEP_CURVE[self.MICROSTEPS*4 - self.currentstep]


		# go to next 'step' and wrap around
		self.currentstep += self.MICROSTEPS * 4
		self.currentstep %= self.MICROSTEPS * 4

		# only really used for microstepping, otherwise always on!
		self.MC._pwm.setPWM(self.PWMA, 0, pwm_a*16)
		self.MC._pwm.setPWM(self.PWMB, 0, pwm_b*16)

		# set up coil energizing!
		coils = [0, 0, 0, 0]

		if (style == Raspi_MotorHAT.MICROSTEP):
			if (self.currentstep >= 0) and (self.currentstep < self.MICROSTEPS):
				coils = [1, 1, 0, 0]
                        elif (self.currentstep >= self.MICROSTEPS) and (self.currentstep < self.MICROSTEPS*2):
				coils = [0, 1, 1, 0]
                        elif (self.currentstep >= self.MICROSTEPS*2) and (self.currentstep < self.MICROSTEPS*3):
				coils = [0, 0, 1, 1]
                        elif (self.currentstep >= self.MICROSTEPS*3) and (self.currentstep < self.MICROSTEPS*4):
				coils = [1, 0, 0, 1]
		else:
			step2coils = [ 	[1, 0, 0, 0], 
				[1, 1, 0, 0],
				[0, 1, 0, 0],
				[0, 1, 1, 0],
				[0, 0, 1, 0],
				[0, 0, 1, 1],
				[0, 0, 0, 1],
				[1, 0, 0, 1] ]
			coils = step2coils[self.currentstep/(self.MICROSTEPS/2)]

		#print "coils state = " + str(coils)
		self.MC.setPin(self.AIN2, coils[0])
		self.MC.setPin(self.BIN1, coils[1])
		self.MC.setPin(self.AIN1, coils[2])
		self.MC.setPin(self.BIN2, coils[3])

		return self.currentstep

	def step(self, steps, direction, stepstyle):
		s_per_s = self.sec_per_step
		lateststep = 0
		
		if (stepstyle == Raspi_MotorHAT.INTERLEAVE):
			s_per_s = s_per_s / 2.0
		if (stepstyle == Raspi_MotorHAT.MICROSTEP):
			s_per_s /= self.MICROSTEPS
			steps *= self.MICROSTEPS

		print s_per_s, " sec per step"

		for s in range(steps):
			lateststep = self.oneStep(direction, stepstyle)
			time.sleep(s_per_s)

		if (stepstyle == Raspi_MotorHAT.MICROSTEP):
			# this is an edge case, if we are in between full steps, lets just keep going
			# so we end on a full step
			while (lateststep != 0) and (lateststep != self.MICROSTEPS):
				lateststep = self.oneStep(dir, stepstyle)
				time.sleep(s_per_s)
		
class Raspi_DCMotor:
	def __init__(self, controller, num):
		self.MC = controller
		self.motornum = num
                pwm = in1 = in2 = 0

                if (num == 0):
                         pwm = 8
                         in2 = 9
                         in1 = 10
                elif (num == 1):
                         pwm = 13
                         in2 = 12
                         in1 = 11
                elif (num == 2):
                         pwm = 2
                         in2 = 3
                         in1 = 4
                elif (num == 3):
                         pwm = 7
                         in2 = 6
                         in1 = 5
		else:
			raise NameError('MotorHAT Motor must be between 1 and 4 inclusive')
                self.PWMpin = pwm
                self.IN1pin = in1
                self.IN2pin = in2

	def run(self, command):
		if not self.MC:
			return
		if (command == Raspi_MotorHAT.FORWARD):
			self.MC.setPin(self.IN2pin, 0)
			self.MC.setPin(self.IN1pin, 1)
		if (command == Raspi_MotorHAT.BACKWARD):
			self.MC.setPin(self.IN1pin, 0)
			self.MC.setPin(self.IN2pin, 1)
		if (command == Raspi_MotorHAT.RELEASE):
			self.MC.setPin(self.IN1pin, 0)
			self.MC.setPin(self.IN2pin, 0)
	def setSpeed(self, speed):
		if (speed < 0):
			speed = 0
		if (speed > 255):
			speed = 255
		self.MC._pwm.setPWM(self.PWMpin, 0, speed*16)

class Raspi_MotorHAT:
	FORWARD = 1
	BACKWARD = 2
	BRAKE = 3
	RELEASE = 4

	SINGLE = 1
	DOUBLE = 2
	INTERLEAVE = 3
	MICROSTEP = 4

	def __init__(self, addr = 0x60, freq = 1600):
		self._i2caddr = addr            # default addr on HAT
		self._frequency = freq		# default @1600Hz PWM freq
		self.motors = [ Raspi_DCMotor(self, m) for m in range(4) ]
		self.steppers = [ Raspi_StepperMotor(self, 1), Raspi_StepperMotor(self, 2) ]
		self._pwm =  PWM(addr, debug=False)
		self._pwm.setPWMFreq(self._frequency)

	def setPin(self, pin, value):
		if (pin < 0) or (pin > 15):
			raise NameError('PWM pin must be between 0 and 15 inclusive')
		if (value != 0) and (value != 1):
			raise NameError('Pin value must be 0 or 1!')
		if (value == 0):
			self._pwm.setPWM(pin, 0, 4096)
		if (value == 1):
			self._pwm.setPWM(pin, 4096, 0)

	def getStepper(self, steps, num):
                if (num < 1) or (num > 2):
                        raise NameError('MotorHAT Stepper must be between 1 and 2 inclusive')
		return self.steppers[num-1]

	def getMotor(self, num):
		if (num < 1) or (num > 4):
			raise NameError('MotorHAT Motor must be between 1 and 4 inclusive')
		return self.motors[num-1]



import jieba
import re
import wikipedia
import os
from playrobot import QA

path = os.path.dirname(os.path.abspath(QA.__file__))
jieba.set_dictionary(path + '/dict.txt')

class wikiQA():
    
    def __init__(self,target):
        self.target = target
        self.head = ''
        wikipedia.set_lang("zh-tw")
    
    def headFinding(self):
        words = jieba.lcut(self.target)
        for word in words:
            if(word != u'是' and word != u'在'):
                self.head += word
            else:
                break
            
    def result(self):
        self.headFinding()
        message = wikipedia.summary(self.head,sentences=1)
        message2 = re.sub(u'[a-zA-Z(),（）「」]','',message)
        print(message)
        return message2


#!/usr/bin/python

import time
import math
from .Raspi_I2C import Raspi_I2C

# ============================================================================
# Raspi PCA9685 16-Channel PWM Servo Driver
# ============================================================================

class PWM :
  # Registers/etc.
  __MODE1              = 0x00
  __MODE2              = 0x01
  __SUBADR1            = 0x02
  __SUBADR2            = 0x03
  __SUBADR3            = 0x04
  __PRESCALE           = 0xFE
  __LED0_ON_L          = 0x06
  __LED0_ON_H          = 0x07
  __LED0_OFF_L         = 0x08
  __LED0_OFF_H         = 0x09
  __ALL_LED_ON_L       = 0xFA
  __ALL_LED_ON_H       = 0xFB
  __ALL_LED_OFF_L      = 0xFC
  __ALL_LED_OFF_H      = 0xFD

  # Bits
  __RESTART            = 0x80
  __SLEEP              = 0x10
  __ALLCALL            = 0x01
  __INVRT              = 0x10
  __OUTDRV             = 0x04

  general_call_i2c = Raspi_I2C(0x00)

  @classmethod
  def softwareReset(cls):
    "Sends a software reset (SWRST) command to all the servo drivers on the bus"
    cls.general_call_i2c.writeRaw8(0x06)        # SWRST

  def __init__(self, address=0x40, debug=False):
    self.i2c = Raspi_I2C(address)
    self.i2c.debug = debug
    self.address = address
    self.debug = debug
    if (self.debug):
      print("Reseting PCA9685 MODE1 (without SLEEP) and MODE2")
    self.setAllPWM(0, 0)
    self.i2c.write8(self.__MODE2, self.__OUTDRV)
    self.i2c.write8(self.__MODE1, self.__ALLCALL)
    time.sleep(0.005)                                       # wait for oscillator
    
    mode1 = self.i2c.readU8(self.__MODE1)
    mode1 = mode1 & ~self.__SLEEP                 # wake up (reset sleep)
    self.i2c.write8(self.__MODE1, mode1)
    time.sleep(0.005)                             # wait for oscillator

  def setPWMFreq(self, freq):
    "Sets the PWM frequency"
    prescaleval = 25000000.0    # 25MHz
    prescaleval /= 4096.0       # 12-bit
    prescaleval /= float(freq)
    prescaleval -= 1.0
    if (self.debug):
      print("Setting PWM frequency to %d Hz" % freq)
      print("Estimated pre-scale: %d" % prescaleval)
    prescale = math.floor(prescaleval + 0.5)
    if (self.debug):
      print("Final pre-scale: %d" % prescale)

    oldmode = self.i2c.readU8(self.__MODE1);
    newmode = (oldmode & 0x7F) | 0x10             # sleep
    self.i2c.write8(self.__MODE1, newmode)        # go to sleep
    self.i2c.write8(self.__PRESCALE, int(math.floor(prescale)))
    self.i2c.write8(self.__MODE1, oldmode)
    time.sleep(0.005)
    self.i2c.write8(self.__MODE1, oldmode | 0x80)

  def setPWM(self, channel, on, off):
    "Sets a single PWM channel"
    self.i2c.write8(self.__LED0_ON_L+4*channel, on & 0xFF)
    self.i2c.write8(self.__LED0_ON_H+4*channel, on >> 8)
    self.i2c.write8(self.__LED0_OFF_L+4*channel, off & 0xFF)
    self.i2c.write8(self.__LED0_OFF_H+4*channel, off >> 8)

  def setAllPWM(self, on, off):
    "Sets a all PWM channels"
    self.i2c.write8(self.__ALL_LED_ON_L, on & 0xFF)
    self.i2c.write8(self.__ALL_LED_ON_H, on >> 8)
    self.i2c.write8(self.__ALL_LED_OFF_L, off & 0xFF)
    self.i2c.write8(self.__ALL_LED_OFF_H, off >> 8)



from .Raspi_PWM_Servo_Driver import PWM

pwm = 0
start = False
def startServo(address):
    global pwm,start
    try:
        pwm = PWM(address)
        pwm.setPWMFreq(50)
        start = True
    except:
        raise BaseException("Bad I2C address")

def setServoPulse(channel, pulse):
    global pwm,start
    if start == False:raise BaseException("please using startServo to set Servo's I2C address")
    pulseLength = 1000000                   # 1,000,000 us per second
    pulseLength /= 50                       # 50 Hz
    pulseLength /= 4096                     # 12 bits of resolution
    pulse /= pulseLength
    pwm.setPWM(channel, 0, int(pulse))


import RPi.GPIO as GPIO
import time

PIN = 26
GPIO.setmode(GPIO.BCM)  
GPIO.setup(PIN,GPIO.IN,GPIO.PUD_UP)
GPIO.setup(12, GPIO.OUT)

def signal_recive():
    if GPIO.input(PIN) == 0:  
        count = 0  
        while GPIO.input(PIN) == 0 and count < 200:  
            count += 1  
            time.sleep(0.00006)  

        count = 0  
        while GPIO.input(PIN) == 1 and count < 80:  
            count += 1  
            time.sleep(0.00006)  

        idx = 0  
        cnt = 0  
        data = [0,0,0,0]  
        for i in range(0,32):  
            count = 0  
            while GPIO.input(PIN) == 0 and count < 15:  
                count += 1  
                time.sleep(0.00006)  

            count = 0  
            while GPIO.input(PIN) == 1 and count < 40:  
                count += 1  
                time.sleep(0.00006)

            if count > 8:  
                data[idx] |= 1<<cnt  
            if cnt == 7:  
                cnt = 0  
                idx += 1  
            else:  
                cnt += 1  
        if data[0]+data[1] == 0xFF and data[2]+data[3] == 0xFF:  
             print("Get the key: 0x%02x" %data[2])  
             return data[2]
        return 0x00


import time
import board
import neopixel

class AdafruitLed():
    def __init__(self,num_pixels):
        self.nowColor = (0,0,0)
        self.transferColor = [(5,0,0),(0,5,0),(-5,0,0),(0,0,5),(0,-5,0),(5,0,0),(0,0,-5),(-5,0,0)]
        self.num_pixels = num_pixels
        self.pixels = neopixel.NeoPixel(board.D18, num_pixels)
    
    def flow(self):
        self.nowColor = (0,0,0)
        for i in self.transferColor:
            for j in range(10):
                self.nowColor += i
                for k in range(self.num_pixels):
                    self.pixels[k] = self.nowColor
                    time.sleep(.3)
    
    def changeAll(self,color):
        for i in range(20):
            self.pixels.fill(self.nowColor - self.nowColor * i / 19)
            time.sleep(.1)
        for i in range(20):
            self.pixels.fill(color * i / 19)
            time.sleep(.1)
        self.nowColor = color
    
    def flowAll(self):
        self.nowColor = (0,0,0)
        for i in self.transferColor:
            for j in range(10):
                self.nowColor += i
                for k in range(self.num_pixels):
                    self.pixels[k] = self.nowColor
                time.sleep(.3)


#!/usr/bin/python
import re
import smbus

# ===========================================================================
# Raspi_I2C Class
# ===========================================================================

class Raspi_I2C(object):

  @staticmethod
  def getPiRevision():
    "Gets the version number of the Raspberry Pi board"
    # Revision list available at: http://elinux.org/RPi_HardwareHistory#Board_Revision_History
    try:
      with open('/proc/cpuinfo', 'r') as infile:
        for line in infile:
          # Match a line of the form "Revision : 0002" while ignoring extra
          # info in front of the revsion (like 1000 when the Pi was over-volted).
          match = re.match('Revision\s+:\s+.*(\w{4})$', line)
          if match and match.group(1) in ['0000', '0002', '0003']:
            # Return revision 1 if revision ends with 0000, 0002 or 0003.
            return 1
          elif match:
            # Assume revision 2 if revision ends with any other 4 chars.
            return 2
        # Couldn't find the revision, assume revision 0 like older code for compatibility.
        return 0
    except:
      return 0

  @staticmethod
  def getPiI2CBusNumber():
    # Gets the I2C bus number /dev/i2c#
    return 1 if Raspi_I2C.getPiRevision() > 1 else 0

  def __init__(self, address, busnum=-1, debug=False):
    self.address = address
    # By default, the correct I2C bus is auto-detected using /proc/cpuinfo
    # Alternatively, you can hard-code the bus version below:
    # self.bus = smbus.SMBus(0); # Force I2C0 (early 256MB Pi's)
    # self.bus = smbus.SMBus(1); # Force I2C1 (512MB Pi's)
    self.bus = smbus.SMBus(busnum if busnum >= 0 else Raspi_I2C.getPiI2CBusNumber())
    self.debug = debug

  def reverseByteOrder(self, data):
    "Reverses the byte order of an int (16-bit) or long (32-bit) value"
    # Courtesy Vishal Sapre
    byteCount = len(hex(data)[2:].replace('L','')[::2])
    val       = 0
    for i in range(byteCount):
      val    = (val << 8) | (data & 0xff)
      data >>= 8
    return val

  def errMsg(self):
    print("Error accessing 0x%02X: Check your I2C address" % self.address)
    return -1

  def write8(self, reg, value):
    "Writes an 8-bit value to the specified register/address"
    try:
      self.bus.write_byte_data(self.address, reg, value)
      if self.debug:
        print("I2C: Wrote 0x%02X to register 0x%02X" % (value, reg))
    except IOError:
      return self.errMsg()

  def write16(self, reg, value):
    "Writes a 16-bit value to the specified register/address pair"
    try:
      self.bus.write_word_data(self.address, reg, value)
      if self.debug:
        print ("I2C: Wrote 0x%02X to register pair 0x%02X,0x%02X" %
         (value, reg, reg+1))
    except IOError:
      return self.errMsg()

  def writeRaw8(self, value):
    "Writes an 8-bit value on the bus"
    try:
      self.bus.write_byte(self.address, value)
      if self.debug:
        print("I2C: Wrote 0x%02X" % value)
    except IOError:
      return self.errMsg()

  def writeList(self, reg, list):
    "Writes an array of bytes using I2C format"
    try:
      if self.debug:
        print("I2C: Writing list to register 0x%02X:" % reg)
        print(list)
      self.bus.write_i2c_block_data(self.address, reg, list)
    except IOError:
      return self.errMsg()

  def readList(self, reg, length):
    "Read a list of bytes from the I2C device"
    try:
      results = self.bus.read_i2c_block_data(self.address, reg, length)
      if self.debug:
        print ("I2C: Device 0x%02X returned the following from reg 0x%02X" %
         (self.address, reg))
        print(results)
      return results
    except IOError:
      return self.errMsg()

  def readU8(self, reg):
    "Read an unsigned byte from the I2C device"
    try:
      result = self.bus.read_byte_data(self.address, reg)
      if self.debug:
        print ("I2C: Device 0x%02X returned 0x%02X from reg 0x%02X" %
         (self.address, result & 0xFF, reg))
      return result
    except IOError:
      return self.errMsg()

  def readS8(self, reg):
    "Reads a signed byte from the I2C device"
    try:
      result = self.bus.read_byte_data(self.address, reg)
      if result > 127: result -= 256
      if self.debug:
        print ("I2C: Device 0x%02X returned 0x%02X from reg 0x%02X" %
         (self.address, result & 0xFF, reg))
      return result
    except IOError:
      return self.errMsg()

  def readU16(self, reg, little_endian=True):
    "Reads an unsigned 16-bit value from the I2C device"
    try:
      result = self.bus.read_word_data(self.address,reg)
      # Swap bytes if using big endian because read_word_data assumes little 
      # endian on ARM (little endian) systems.
      if not little_endian:
        result = ((result << 8) & 0xFF00) + (result >> 8)
      if (self.debug):
        print("I2C: Device 0x%02X returned 0x%04X from reg 0x%02X" % (self.address, result & 0xFFFF, reg))
      return result
    except IOError:
      return self.errMsg()

  def readS16(self, reg, little_endian=True):
    "Reads a signed 16-bit value from the I2C device"
    try:
      result = self.readU16(reg,little_endian)
      if result > 32767: result -= 65536
      return result
    except IOError:
      return self.errMsg()

if __name__ == '__main__':
  try:
    bus = Raspi_I2C(address=0)
    print("Default I2C bus is accessible")
  except:
    print("Error accessing default I2C bus")



import speech_recognition as sr
import pyaudio
import wave

CHUNK = 1024
RATE = 16000
RECORD_SECONDS = 5
FORMAT = pyaudio.paInt16
CHANNELS = 1

def setProperty(chunk = 1024,rate = 16000,rec = 5):
    global CHUNK,RATE,RECORD_SECONDS
    CHUNK = chunk
    RATE = rate
    RECORD_SECONDS = rec
    
def recordWave():
    global CHUNK,RATE,RECORD_SECONDS,FORMAT,CHANNELS
    pa = pyaudio.PyAudio()
    stream = pa.open(format=FORMAT,
                     channels=CHANNELS,
                     rate=RATE,
                     input=True,
                     frames_per_buffer=CHUNK)
    print('Recording...')
    buffer = []
    for i in range(0, int(RATE/CHUNK*RECORD_SECONDS)):
        audio_data = stream.read(CHUNK)
        buffer.append(audio_data)
    print('Record Done')
    stream.stop_stream()
    stream.close()
    pa.terminate()
    wf = wave.open('record.wav', 'wb')
    wf.setnchannels(CHANNELS)
    wf.setsampwidth(pa.get_sample_size(FORMAT))
    wf.setframerate(RATE)
    wf.writeframes(b''.join(buffer))
    wf.close()
    

def stt():
    recordWave()
    r = sr.Recognizer()
    with sr.AudioFile("record.wav") as source:
        sound = r.listen(source)
    try:
        #語音辨識轉成文字
        print('Recognizing')
        target = r.recognize_google(sound, language="zh-TW")
        print(target)
        return target
    except sr.UnknownValueError:
        print("Google Speech Recognition could not understand audio")
    except sr.RequestError as e:
        print("No response from Google Speech Recognition service: {0}".format(e))



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

setup(
    name='playrobot',
    version='0.0.27',
    description=(
        'lib for playrobot'
    ),
    long_description=open('README.rst').read(),
    author='ckshadow',
    author_email='99robot@playrobot.com',
    maintainer='ckshadow',
    maintainer_email='99robot@playrobot.com',
    license='BSD License',
    packages=find_packages(),
    platforms=["all"],
    url='',
	package_data={
        # If any package contains *.txt files, include them:
        '': ['*.*']},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'jieba',
        'gTTS',
        'wikipedia',
        'pygame',
        'wave',
        'pyaudio',
        'paho-mqtt',
        'SpeechRecognition',
    ],
)



from aip import AipSpeech,AipOcr
import pyaudio
import wave
from pygame import mixer

APP_ID = ''
API_KEY = ''
SECRET_KEY = ''
client = ''
CHUNK = 1024
RATE = 16000
RECORD_SECONDS = 5


def setProperty(ID = '',AK = '', SK = '',chunk = 1024 ,rate = 16000,rec = 5):
    global APP_ID,API_KEY,SECRET_KEY,client,CHUNK,RATE,RECORD_SECONDS,client
    APP_ID = ID
    API_KEY = AK
    SECRET_KEY = SK
    CHUNK = chunk
    RATE = rate
    RECORD_SECONDS = rec
    client = AipSpeech(APP_ID,API_KEY,SECRET_KEY)

def get_file_content(filePath):
    with open(filePath, 'rb') as fp:
        return fp.read()
    
def recordWave():
    global CHUNK,RATE,RECORD_SECONDS
    FORMAT = pyaudio.paInt16
    CHANNELS = 1
    pa = pyaudio.PyAudio()
    stream = pa.open(format=FORMAT,
                     channels=CHANNELS,
                     rate=RATE,
                     input=True,
                     frames_per_buffer=CHUNK)
    print('Recording...')
    buffer = []
    for i in range(0, int(RATE/CHUNK*RECORD_SECONDS)):
        audio_data = stream.read(CHUNK)
        buffer.append(audio_data)
    print('Record Done')
    stream.stop_stream()
    stream.close()
    pa.terminate()
    wf = wave.open('record.wav', 'wb')
    wf.setnchannels(CHANNELS)
    wf.setsampwidth(pa.get_sample_size(FORMAT))
    wf.setframerate(RATE)
    wf.writeframes(b''.join(buffer))
    wf.close()

def STT():
    global APP_ID,API_KEY,SECRET_KEY,client
    if APP_ID == '' or API_KEY == '' or SECRET_KEY == '':
        raise BaseException('please using setProperty to set API Property')
    recordWave()
    result = str(client.asr(get_file_content('record.wav'), 'wav', 16000, {'dev_pid': 1536,})['result'])
    result = result[2:len(result)-2]
    return result

def TTS(target,sound = 1):
    if type(target) != str:
        raise BaseException('input not a string')
    result  = client.synthesis(target, 'zh', 1, {'vol': 5,'spd': 3,'per': sound})
    if not isinstance(result, dict):
        with open('test.mp3', 'wb') as f:
            f.write(result)
    mixer.init()
    mixer.music.load('test.mp3')
    mixer.music.play()
    while mixer.music.get_busy() == True:
        continue
    mixer.music.stop()
    mixer.quit()

def imageToText(target):
    global APP_ID,API_KEY,SECRET_KEY
    client = AipOcr(APP_ID, API_KEY, SECRET_KEY)
    if APP_ID == '' or API_KEY == '' or SECRET_KEY == '':
        raise BaseException('please using setProperty to set API Property')
    return client.basicGeneral(target);